import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class Font {

public static final int STYLE_NORMAL = 1;
public static final int STYLE_MRSATURN = 2;

private static final int numChars = 75;

int index;
int h;
int w[] = new int[numChars];
Image c[];

public Font(int index) {
	this.index = index;
	loadFont(index);
	buildFont();
}

public void loadFont(int index) {

	StringTokenizer tokens; //File input variables
	String inFileName;			//
	InputStreamReader inStream;		//
	BufferedReader in;			//
	String fileLine;				//

	try {
		inStream = new InputStreamReader(ClassLoader.getSystemResource("Font/"+index+"f.tls").openStream());
		in = new BufferedReader(inStream);

		//Font height
		fileLine = in.readLine();
		tokens = new StringTokenizer(fileLine);
		h = Integer.parseInt(tokens.nextToken());

		//Uppercase
		fileLine = in.readLine();
		tokens = new StringTokenizer(fileLine);
		for(int i=17;i<43;i++)
			w[i] = Integer.parseInt(tokens.nextToken());

		//Lowercase
		fileLine = in.readLine();
		tokens = new StringTokenizer(fileLine);
		for(int i=49;i<75;i++)
			w[i] = Integer.parseInt(tokens.nextToken());

		//Numbers
		fileLine = in.readLine();
		tokens = new StringTokenizer(fileLine);
		for(int i=0;i<10;i++)
			w[i] = Integer.parseInt(tokens.nextToken());

		//Space
		w[47] = 6;

		in.close();
	}
	catch (IOException e) {
		System.out.println(e.getMessage());
	}
}

public void buildFont() {
	ImageIcon img;
	Image chars;
	Graphics g;

	c = new Image[numChars];
	img = new ImageIcon(ClassLoader.getSystemResource("Font/"+index +"f.tlr"));
	chars = img.getImage();

	//Uppercase
	for (int i=17, cWidth=0; i<43; i++) {
		c[i] = new BufferedImage(w[i],h, BufferedImage.TYPE_INT_RGB);
		g = c[i].getGraphics();
		g.drawImage(
			chars,
			0,0,w[i],h,
			cWidth,0,
			cWidth+w[i],h,
			null);
		cWidth += w[i];
	}

	//Lowercase
	for (int i=49, cWidth=0; i<75; i++) {
		c[i] = new BufferedImage(w[i],h, BufferedImage.TYPE_INT_RGB);
		g = c[i].getGraphics();
		g.drawImage(
			chars,
			0,0,w[i],h,
			cWidth,h,
			cWidth+w[i],h*2,
			null);
		cWidth += w[i];
	}

	//Numbers
	for (int i=0, cWidth=0; i<10; i++) {
		c[i] = new BufferedImage(w[i],h, BufferedImage.TYPE_INT_RGB);
		g = c[i].getGraphics();
		g.drawImage(
			chars,
			0,0,w[i],h,
			cWidth,h*2,
			cWidth+w[i],h*3,
			null);
		cWidth += w[i];
	}

	//Space
	c[47] = new BufferedImage(w[47],h, BufferedImage.TYPE_INT_RGB);
	g = c[47].getGraphics();
	g.setColor(new Color(16,16,16));
	g.fillRect(0,0,w[47],h);
}

public Image getString(String s) {

	Image sImg;
	Graphics g;

	char[] sChar = s.toCharArray();
	int sWidth = 0, cIndex;

	for(int i=0;i<sChar.length;i++) {
		if(sChar[i]-'0' == -16)
			sWidth += w[47];
		else
			sWidth += w[sChar[i]-'0'];
	}

	sImg = new BufferedImage(sWidth,h,BufferedImage.TYPE_INT_RGB);
	g = sImg.getGraphics();
	g.setColor(new Color(16,16,16));

	for (int i=0, cWidth=0; i<sChar.length; i++) {
		cIndex = sChar[i]-'0';

		if(cIndex == -16)	cIndex = 47;

		g.drawImage(
			c[cIndex],
			cWidth,0,cWidth+w[cIndex],h,
			0,0,
			w[cIndex],h,
			null);
		cWidth += w[cIndex];
	}
	return sImg;
}

public Image getString(String s, boolean filter) {
	return new Label().createImage(
		new FilteredImageSource(getString(s).getSource(),
		new filterMask()));
}

}//Font