import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class Map extends Main{

Graphics G;

private final static int tS = 16; //size of tiles

public boolean isReady = false; //if the Map is ready for drawing yet
int index;		//index of Map file
int sizeX; 		//number of tiles in X direction
int sizeY; 		//number of tiles in Y direction

int m[][];		//int array representing tiles on map
int o[][]; 		//int array representing overdraw tiles on map
int h[][];		//hit collision map

int numTiles;  //total number of unique tiles used on map
Image[] tile;  //Image array storing all tiles used in map
Image tileSet; //Image storing tiles used
Image MAP;		 //built map image
Image map;		 //updated map image


Vector gate = new Vector();						//Gateway vector
Vector s = new Vector();							//Sprite vector
static Sprite spr = new Sprite();			//Used for map updates
static Rectangle b = new Rectangle(); //Used for hit detection

/* Map - constructor, initial state
 */
public Map() {}

/* Map - constructor, loads and initializes Map objects
 * @param mapNum
 *				-index of Map file to be loaded
 */
public Map(int mapNum) {
	index = mapNum;

	StringTokenizer tokens; //File input variables
	String inFileName;			//
	InputStreamReader inStream;		//
	BufferedReader in;			//
	String fileLine;				//

	try {
		inStream = new InputStreamReader(ClassLoader.getSystemResource("Maps/" + (Integer.toString(index) + "t.tls")).openStream());
		in = new BufferedReader(inStream);
		fileLine = in.readLine();
		tokens = new StringTokenizer(fileLine);

		//Dimensions of map
		sizeX = Integer.parseInt(tokens.nextToken());
		sizeY = Integer.parseInt(tokens.nextToken());

		//get numTiles
		fileLine = in.readLine();
		numTiles = Integer.parseInt(fileLine);
		tile = new BufferedImage[numTiles];

		//Construct map array
		m = new int[sizeY][sizeX];
		for (int i=0; i<sizeY; i++) {
			fileLine = in.readLine();
			tokens = new StringTokenizer(fileLine);
			for (int j=0; j<sizeX; j++)
				m[i][j] = Integer.parseInt(tokens.nextToken());
		}

		//Load collision map
		h = new int[sizeY][sizeX];
		for (int i=0; i<sizeY; i++) {
			fileLine = in.readLine();
			tokens = new StringTokenizer(fileLine);
			for (int j=0; j<sizeX; j++)
				h[i][j] = Integer.parseInt(tokens.nextToken());
		}

		//Load gateways
		fileLine = in.readLine();
		while(fileLine!=null){
		tokens = new StringTokenizer(fileLine);
		gate.add(new Gateway(Integer.parseInt(tokens.nextToken()),
												 Integer.parseInt(tokens.nextToken()),
												 Integer.parseInt(tokens.nextToken()),
												 Integer.parseInt(tokens.nextToken())));
		fileLine = in.readLine();
		}

		in.close();

	}
	catch (IOException e) {
		System.out.println(e.getMessage());
	}

} //Map


/* buildMap - constructs final Map from tileset
 * @return void
 */
public void buildMap() {

	Graphics g;
	ImageIcon img;

	//Init MAP dimensions and graphics context
	MAP = new BufferedImage(sizeX*tS, sizeY*tS, BufferedImage.TYPE_INT_RGB);
	map = new BufferedImage(sizeX*tS, sizeY*tS, BufferedImage.TYPE_INT_RGB);
	G = MAP.getGraphics();

	//Load tileSet
	img = new ImageIcon(ClassLoader.getSystemResource("Maps/"+ index + "t.tlr"));
	tileSet = img.getImage();

	//Load the tiles
	for (int i=0; i<numTiles; i++) {
		tile[i] = new BufferedImage(tS,tS, BufferedImage.TYPE_INT_RGB);
		g = tile[i].getGraphics();
		g.drawImage(
			tileSet,
			0,0,tS,tS,
			(i*tS) % (32*tS),(i/32)*tS,(i*tS) % (32*tS) + tS,(i/32) * tS + tS,
			null);
	}

	//Build the map
	for (int i=0; i<sizeY; i++)
		for(int j=0; j<sizeX; j++)
			G.drawImage(tile[m[i][j]],j*tS,i*tS,tS,tS,null);

/*G.setColor(Color.RED);
	for (int i=0; i<sizeY; i++) {
		G.drawLine(0,i*tS,sizeX*tS,i*tS);
		for(int j=0; j<sizeX; j++)
		G.drawLine(j*tS,0,j*tS,sizeY*tS);
	}*/

	G = map.getGraphics();

//isReady = !tracker.isErrorAny();

	loadSprites();

}//buildMap

public void loadSprites() {

	if(index == 1) {
		s.add(new Sprite(1,150,144));
		s.add(new Sprite(2,182,144));
		s.add(new Sprite(3,214,144));
		s.add(new Sprite(4,246,144));
	}

}

public void loadSprites(Vector sprites) {

	for(int i=0;i<sprites.size();i++)
		s.add(i,sprites.get(i));

}

public void updateMap() {
	G.drawImage(MAP,0,0,null);
	for(int i=(s.size()-1);i>=0;i--) {
		spr = (Sprite)s.get(i);
		G.drawImage(spr.frame(),spr.getX(),spr.getY(),null);

/*	b = spr.getBox();
		G.setColor(Color.BLUE);
		G.drawRect(spr.getBX(),spr.getBY(),b.width,b.height);*/

	}
}

/* checkCollision - checks to see if bounding box of sprite intersects a wall tile
 * @param index
 *				-index of Sprite to be checked for collision
 * @return void
 */
public void checkCollision(int index) {
	try{
	spr = (Sprite)s.get(index);
	b = spr.getBox();
	for(int i = b.y/tS, H = (b.y+b.height)/tS; i <= H; i++){
	for(int j = b.x/tS, W = (b.x+b.width )/tS; j <= W; j++){
	if(h[i][j] != 0) {
	switch(h[i][j]) {
/*
	//Walls
	case 1://full square
		switch(spr.getDir()){
		case 1://down
			spr.setLoc(spr.getBX(),i*tS-spr.getBH()-1);
			break;
		case 2://up
			spr.setLoc(spr.getBX(),i*tS+1);
			break;
		case 3://left
			spr.setLoc(j*tS+1,spr.getBY());
			break;
		case 4://right
			spr.setLoc(j*tS-spr.getBW()-1,spr.getBY());
			break;
		case 5://down-left
			if(spr.pX >= (j+1)*tS)
				spr.setLoc((j+1)*tS+1,spr.getBY());
			else
				spr.setLoc(spr.getBX(),i*tS-spr.getBH()-1);
			break;
		case 6://down-right
			if(spr.pX+spr.getBW() <= j*tS)
				spr.setLoc(j*tS-spr.getBW()-1,spr.getBY());
			else
				spr.setLoc(spr.getBX(),i*tS-spr.getBH()-1);
			break;
		case 7://up-left
			if(spr.pX >= (j+1)*tS)
				spr.setLoc((j+1)*tS+1,spr.getBY());
			else
				spr.setLoc(spr.getBX(),(i-1)*tS+1);
			break;
		case 8://up-right
			if(spr.pX >= (j+1)*tS)
				spr.setLoc(j*tS-spr.getBW()-1,spr.getBY());
			else
				spr.setLoc(j*tS-spr.getBW()-1,spr.getBY());
			break;
		}
		break;
	case 2://top left triangle
		if((spr.getBX() < (j*tS+(tS-spr.getBY()%tS))) &&
			 (spr.getBY() < (i*tS+(tS-spr.getBX()%tS))))
			spr.setLoc((j)*tS+((spr.getBX()%tS+(tS-spr.getBY()%tS))/2),(i)*tS+tS-((spr.getBX()%tS+(tS-spr.getBY()%tS))/2));
		break;
	case 3://top right triangle
		if((spr.getBX()+spr.getBW() > (j*tS+(spr.getBY()%tS))) &&
			 (spr.getBY() < (i*tS+(spr.getBX()%tS))))
			spr.setLoc((j)*tS+((spr.getBX()%tS+(tS-spr.getBY()%tS))/2)-spr.getBW(),(i)*tS+((spr.getBX()%tS+(spr.getBY()%tS))/2));
		break;
	case 4://bottom left triangle
		switch(spr.getDir()){
		case 1://down
			break;
		case 3://left
			break;
		case 5://down-left
			break;
		}
		break;
	case 5://bottom right triangle
		switch(spr.getDir()){
		case 1://down
			break;
		case 4://right
			break;
		case 6://down-right
			break;
		}
		break;
	case 6://bottom wall
		while(!done) {
		if(!(spr.getBY()+spr.getBH() > (i+1)*tS && spr.getBY() < (i+1)*tS))
			done = true;
		else {
		k++;
		if(k>1) spr.setLoc(spr.getX()+1,(i+1)*tS-spr.bH-spr.bY);
		switch(spr.getDir()){
		case 1://down
			spr.setLoc(spr.getX(),(i+1)*tS-spr.bH-spr.bY-1);
			break;
		case 5://down-left
			spr.setLoc(spr.pX,(i+1)*tS-spr.bH-spr.bY-1);
			break;
		case 6://down-right
			spr.setLoc(spr.pX,(i+1)*tS-spr.bH-spr.bY-1);
			break;
		}}}
		break;
	case 7://top wall
		switch(spr.getDir()){
		case 2://up
			break;
		case 7://up-left
			break;
		case 8://up-right
			break;
		}
		break;
	case 8://left wall
		switch(spr.getDir()){
		case 3://left
			break;
		case 5://down-left
			break;
		case 7://up-left
			break;
		}
		break;
	case 9://right wall
		switch(spr.getDir()){
		case 4://right
			break;
		case 6://down-right
			break;
		case 8://up-right
			break;
		}
		break;
*/
	//Gateways
	default:

	if(h[i][j] >= 100)
		if((spr.getBX() < (j*tS+(tS-spr.getBY()%tS))) &&
			 (spr.getBY() < (i*tS+(tS-spr.getBX()%tS)))) {
			useGate((Gateway)gate.get(h[i][j]%100),s);
		}
		break;
	}}}}
}
catch(Exception e) {
}
}//checkCollision

public void checkSprites() {
	for(int i=0; i < s.size(); i++) {
		((Sprite)s.get(i)).check();
		checkCollision(i);
	}
}

/* getMap - accessor, returns the MAP image
 * @return Image
 */
public final Image getMap() {
	return map;
}//getMap

public final Image getMap(boolean getOriginal) {
	return MAP;
}//getMap

public final int getNumSprites() {
	return s.size();
}

public final Sprite getS(int index) {
	return (Sprite)s.get(index);
}

public final int getSX(int index) {
	return ((Sprite)s.get(index)).getX();
}

public final int getSY(int index) {
	return ((Sprite)s.get(index)).getY();
}

}//Map
