import java.awt.*;
import java.awt.event.*;
import java.io.*;

class MapPainter extends Frame implements KeyListener, MouseListener, MouseMotionListener, Runnable {


static final int WIDTH = 512;		//Viewable width
static final int HEIGHT = 448;	//Viewable height
static int X, Y, mX, mY;				//(0,0) coords relative to Insets


static boolean running = true;
static boolean mouseDown = false;
static boolean loading = true;

static Insets I;
static int tileType = 0,tX,tY;
static int tMap[][];

static Thread thread;
static MapPainter M;
static Graphics G;
static Image IMG;

static Map map = new Map();
static int currMap;

public MapPainter(String title) {
	super(title);
	this.setResizable(false);
	this.setVisible(true);
	I = this.getInsets();
	this.setSize(WIDTH + I.left + I.right,HEIGHT + I.top + I.bottom);
	X = I.left;
	Y = I.top;

	IMG = createImage(WIDTH, HEIGHT);
	G = IMG.getGraphics();
	addKeyListener(this);
	addMouseListener(this);
	addMouseMotionListener(this);
} //Main

public static void main(String [] args) {

	try {
	currMap = Integer.parseInt(args[0]);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	currMap = 6;
	}

	M = new MapPainter("MapPainter");

	thread = new Thread(M);
	thread.start();

}//main

public void run() {
	map = new Map(currMap);
	M.add(map);
	map.buildMap();
	M.setSize((map.sizeX*16) + I.left + I.right,(map.sizeY*16) + I.top + I.bottom);
	tMap = new int[map.sizeY][map.sizeX];
	IMG = createImage(map.sizeX*16, map.sizeY*16);
	G = IMG.getGraphics();
	loading = false;
	while(running) {
	if(mouseDown) {
	try {
	tMap[tY][tX] = tileType;
	}
	catch (Exception e){}}
	M.repaint();
	}
}//run

public void mouseClicked(MouseEvent e){
	//System.out.println(tMap[tY][tX]);
	System.out.println(tX + "," + tY);
}

public void mouseDragged(MouseEvent e){
	mX = e.getX() - I.left;
	mY = e.getY() - I.top;
	tX = mX/16;
	tY = mY/16;
	if(mouseDown) {
	try {tMap[tY][tX] = tileType;
	}
	catch (Exception err){}}
}

public void mouseMoved(MouseEvent e){
	mX = e.getX() - I.left;
	mY = e.getY() - I.top;
	tX = mX/16;
	tY = mY/16;
	if(mouseDown) {
	try {tMap[tY][tX] = tileType;
	}
	catch (Exception err){}}
}

public void mousePressed(MouseEvent e){
	mouseDown = true;
}
public void mouseReleased(MouseEvent e){
	mouseDown = false;
}

public void keyPressed(KeyEvent evt) {
	tileType = evt.getKeyCode() - '0';
	M.setTitle("MapPainter [tileType = " + tileType + "]");

	if(tileType == -38) {
		printTileMap();
	}
	if(tileType == -21) {
		running = false;
		System.exit(0);
	}

}


public void pause(int time) {
	try { Thread.sleep(time); }
	catch (InterruptedException e) {}
}//pause

public void paint(Graphics g) {
	if(loading) return;
	try{
	G.drawImage(map.getMap(true),0,0,this);
	G.setColor(Color.RED);

	for(int i=0;i<map.sizeX;i++)
		G.drawLine(16*i,0,16*i,16*map.sizeY);

	for(int i=0;i<map.sizeY;i++)
		G.drawLine(0,16*i,16*map.sizeX,16*i);

	G.setColor(Color.BLUE);

	for(int i=0;i<map.sizeY;i++) {
	for(int j=0;j<map.sizeX;j++) {
	if(tMap[i][j] > 0) {
	switch(tMap[i][j]){
	case 1:	//square
		G.drawRect(j*16,i*16,16,16);
		G.drawRect(j*16+1,i*16+1,14,14);
		break;
	case 2: //top left triangle
		G.drawLine(j*16,i*16,j*16+16,i*16);
		G.drawLine(j*16,i*16+1,j*16+16,i*16+1);

		G.drawLine(j*16,i*16,j*16,i*16+16);
		G.drawLine(j*16+1,i*16,j*16+1,i*16+16);

		G.drawLine(j*16+16,i*16,j*16,i*16+16);
		G.drawLine(j*16+16,i*16-1,j*16,i*16+15);
		break;
	case 3: //top right triangle
		G.drawLine(j*16,i*16,j*16+16,i*16);
		G.drawLine(j*16,i*16+1,j*16+16,i*16+1);

		G.drawLine(j*16+16,i*16,j*16+16,i*16+16);
		G.drawLine(j*16+15,i*16,j*16+15,i*16+16);

		G.drawLine(j*16,i*16,j*16+16,i*16+16);
		G.drawLine(j*16+1,i*16,j*16+16,i*16+15);
		break;
	case 4:
		G.drawLine(j*16,i*16,j*16,i*16+16);
		G.drawLine(j*16+1,i*16,j*16+1,i*16+16);

		G.drawLine(j*16,i*16+16,j*16+16,i*16+16);
		G.drawLine(j*16,i*16+15,j*16+16,i*16+15);

		G.drawLine(j*16,i*16,j*16+16,i*16+16);
		G.drawLine(j*16+1,i*16,j*16+16,i*16+15);
		break;
	case 5:
		G.drawLine(j*16+16,i*16,j*16+16,i*16+16);
		G.drawLine(j*16+15,i*16,j*16+15,i*16+16);

		G.drawLine(j*16,i*16+16,j*16+16,i*16+16);
		G.drawLine(j*16,i*16+15,j*16+16,i*16+15);

		G.drawLine(j*16+16,i*16,j*16,i*16+16);
		G.drawLine(j*16+16,i*16-1,j*16,i*16+15);
		break;
	case 6:
		G.drawLine(j*16,i*16+16,j*16+16,i*16+16);
		G.drawLine(j*16,i*16+14,j*16+16,i*16+14);
		break;
	case 7:
		G.drawLine(j*16,i*16,j*16+16,i*16);
		G.drawLine(j*16,i*16+2,j*16+16,i*16+2);
		break;
	case 8:
		G.drawLine(j*16,i*16,j*16,i*16+16);
		G.drawLine(j*16+2,i*16,j*16+2,i*16+16);
		break;
	case 9:
		G.drawLine(j*16+16,i*16,j*16+16,i*16+16);
		G.drawLine(j*16+14,i*16,j*16+14,i*16+16);
		break;
	}}}}
}catch(Exception e){}
	g.drawImage(IMG,X,Y,this);
}//paint

public void update(Graphics g) {
	paint(g);
}//update

public void keyReleased(KeyEvent evt) {}
public void keyTyped(KeyEvent evt) {}
public void mouseEntered(MouseEvent e){}
public void mouseExited(MouseEvent e){}


public void printTileMap() {
	String s;
	try{
	FileWriter outStream = new FileWriter(currMap + ".tls", true);
	PrintWriter out = new PrintWriter(outStream);

	for(int i=0;i<map.sizeY;i++) {
	s = "";
	for(int j=0;j<map.sizeX;j++) {
	s += tMap[i][j] + " ";
	}
	out.println(s);
	}
	out.close();

	}
	catch (IOException e) {}
	M.setTitle("Output printed to " + currMap + ".tls");
}

}//Main